# unit-system

[![GitHub license](https://img.shields.io/github/license/noah1510/unit-system.svg)](https://github.com/noah1510/unit-system/blob/master/License)
[![stable release](https://img.shields.io/github/v/release/noah1510/unit-system.svg)](https://GitHub.com/noah1510/unit-system/releases/)
[![arduino-library-badge](https://www.ardu-badge.com/badge/unit-system.svg?)](https://www.ardu-badge.com/unit-system)

**Please note that the project is still in the early development state and some things might not work as expected!**

**As of version 0.6.0 all the source code is generated by python scripts.**
**Along with that there were more changes which causes older version to break.**
**Scroll down to see more information on how to upgrade to the new version**

This is an implementation of the SI unit system for C++ development.
All units, combinations, literals and consants are generated from templates.
If you feel like anything is missing just add it to the build system and open a pull request.


Although this library is focued on SI units it can be used to work with imperial units, but there will ***NEVER*** be upstream support for it.
If you want to use imperial units you can figure out how to convert and add literals by yourself.

## Usage examples

```c++
#include "unit_system.hpp"
#include <iostream>

using namespace sakurajin::unit_system;
using namespace sakurajin::unit_system::literals;

int main(){
    const auto t2 = 10.0_h;
    const auto s2 = 250.0_km;
    
    std::cout << s2 << " driven in " << t2 << " is an average speed of:" << s2/t2 << std::endl;
}

```

## Including the unit-system library

Depending on your build system you might have to figure out how to include and link this library by yourself.

### meson

unit-system is written in C++ with the meson build system.
Integrating this library is very easy.
Just use the wrapdb to download the version you need, and then you can add the library as a dependency object.

## Units that are currently supported

* unit_t -> an unit template that is the basis of all of the units
* time_si -> time with seconds as base unit
* length -> length with meter as base unit
* mass -> mass with kg as base unit
* temperature -> temperatur with K as base unit
* amount -> amount of substance with mole as base unit
* electric_current -> electric current with Ampere as base unit
* luminous_intensity -> luminous intensity with candela as base unit
* area -> area with `m^2` as base unit
* speed -> speed with `m / s` as base unit
* acceleration -> acceleration with `m / s^2` as base unit
* momentum -> momentum with `kg * m / s` as base unit
* force -> force with `kg * m / s^2` (Newton) as base unit
* energy -> energy with `kg * m^2 / s^2` (Joules) as base unit
* power -> power with `kg * m^2 / s^3` (Watts) as base unit

## Upgrade Instructions

###  from version pre 0.6.0 to 0.6.x

As of version 0.6.0 there are no longer sub namespaces for base and common units.
Just change `sakurajin::unit_system::base` and `sakurajin::unit_system::common` to `sakurajin::unit_system`.
This also affects literals and constants so they are now all under `sakurajin::unit_system::literals` and `sakurajin::unit_system::constants`.

In addition to that a lot of data was moved around in the headers.
Because of that it is no longer recommended to include only some parts of the library.
So instead of including the units directly just include the main header for this library `#include "unit_system.hpp"`.

